module loggamma_mod
use dal

contains

FUNCTION logGamma(xvalue) RESULT(fn_val)

!     ALGORITHM AS245  APPL. STATIST. (1989) VOL. 38, NO. 2

!     Calculation of the logarithm of the gamma function

! ELF90-compatible version by Alan Miller
! Latest revision - 29 November 1997

! N.B. Argument IFAULT has been removed

IMPLICIT NONE

REAL (double), INTENT(IN) :: xvalue
REAL (double)             :: fn_val

! Local variables
REAL (double) :: x, x1, x2, y

!     Coefficients of rational functions

REAL (double), PARAMETER :: r1(9) = (/ -2.66685511495_double, -24.4387534237_double,  &
                                   -21.9698958928_double,  11.1667541262_double,  &
                                    3.13060547623_double,  0.607771387771_double, &
                                    11.9400905721_double,  31.4690115749_double,  &
                                    15.2346874070_double /)
REAL (double), PARAMETER :: r2(9) = (/ -78.3359299449_double, -142.046296688_double,  &
                                    137.519416416_double,  78.6994924154_double,  &
                                    4.16438922228_double,  47.0668766060_double,  &
                                    313.399215894_double,  263.505074721_double,  &
                                    43.3400022514_double /)
REAL (double), PARAMETER :: r3(9) = (/ -2.12159572323E5_double,  2.30661510616E5_double,  &
                                    2.74647644705E4_double, -4.02621119975E4_double,  &
                                   -2.29660729780E3_double, -1.16328495004E5_double,  &
                                   -1.46025937511E5_double, -2.42357409629E4_double,  &
                                   -5.70691009324E2_double /)
REAL (double), PARAMETER :: r4(5) = (/ 0.279195317918525_double, 0.4917317610505968_double, &
                                   0.0692910599291889_double, 3.350343815022304_double, &
                                   6.012459259764103_double /)

!     Fixed constants

REAL (double), PARAMETER :: alr2pi = 0.918938533204673_double, four = 4.0_double,  &
                        half = 0.5_double, one = 1._double, onep5 = 1.5_double,   &
                        twelve = 12._double, zero = 0._double

!     Machine-dependant constants.
!     A table of values is given at the top of page 399 of the paper.
!     These values are for the IEEE double-precision format for which
!     B = 2, t = 53 and U = 1023 in the notation of the paper.

REAL (double), PARAMETER :: xlge = 5.10E6_double, xlgst = HUGE(1.0_double)

x = xvalue
fn_val = zero

!     Test for valid function argument

IF (x >= xlgst) THEN
  WRITE(*, *) 'AS 245: Argument x too large'
  RETURN
END IF
IF (x <= zero) THEN
 
  RETURN
END IF

!     Calculation for 0 < X < 0.5 and 0.5 <= X < 1.5 combined

IF (x < onep5) THEN
  IF (x < half) THEN
    fn_val = -LOG(x)
    y = x + one
    
!     Test whether X < machine epsilon
    
    IF (y == one) RETURN
  ELSE
    fn_val = zero
    y = x
    x = (x - half) - half
  END IF
  fn_val = fn_val + x * ((((r1(5)*y + r1(4))*y + r1(3))*y + r1(2))*y + r1(1)) / &
                    ((((y + r1(9))*y + r1(8))*y+ r1(7))*y + r1(6))
  RETURN
END IF

!     Calculation for 1.5 <= X < 4.0

IF (x < four) THEN
  y = (x - one) - one
  fn_val = y * ((((r2(5)*x + r2(4))*x + r2(3))*x + r2(2))*x + r2(1)) /  &
               ((((x + r2(9))*x + r2(8))*x + r2(7))*x+ r2(6))
  RETURN
END IF

!     Calculation for 4.0 <= X < 12.0

IF (x < twelve) THEN
  fn_val = ((((r3(5)*x + r3(4))*x + r3(3))*x + r3(2))*x + r3(1)) /  &
           ((((x + r3(9))*x + r3(8))*x + r3(7))*x + r3(6))
  RETURN
END IF

!     Calculation for X >= 12.0

y = LOG(x)
fn_val = x * (y - one) - half * y + alr2pi
IF (x > xlge) RETURN
x1 = one / x
x2 = x1 * x1
fn_val = fn_val + x1 * ((r4(3)*x2 + r4(2))*x2 + r4(1)) /  &
         ((x2 + r4(5))*x2 + r4(4))


END FUNCTION logGamma


end module loggamma_mod
